@@ -6,17 +6,17 @@
    :copyright: Copyright 2014 Marshall Ward, see AUTHORS for details.
    :license: Apache License, Version 2.0, see LICENSE for details.
 """
-__version__ = '0.10.1'
+__version__ = '0.10.2'
 
 from f90nml.parser import Parser
 
 # Legacy API functions
 
-def read(nml_fname, verbose=False):
+def read(nml_fname):
     """Parse a Fortran 90 namelist file (data.nml) and store its contents.
 
     >>> nml = f90nml.read('data.nml')"""
-    return Parser(verbose).read(nml_fname)
+    return Parser().read(nml_fname)
 
 def write(nml, nml_fname, force=False):
     """Output namelist (nml) to a Fortran 90 namelist file (data.nml).
@@ -24,8 +24,8 @@ def write(nml, nml_fname, force=False):
     >>> f90nml.write(nml, 'data.nml')"""
     nml.write(nml_fname, force)
 
-def patch(nml_fname, nml_patch, out_fname=None, verbose=False):
+def patch(nml_fname, nml_patch, out_fname=None):
     """Create a new namelist based on an input namelist and reference dict.
 
     >>> f90nml.patch('data.nml', nml_patch, 'patched_data.nml')"""
-    return Parser(verbose).read(nml_fname, nml_patch, out_fname)
+    return Parser().read(nml_fname, nml_patch, out_fname)
