@@ -4,6 +4,7 @@ import unittest
 
 sys.path.insert(1, '../')
 import f90nml
+from f90nml.fpy import f90repr, pybool
 
 class Test(unittest.TestCase):
 
@@ -16,6 +17,12 @@ class Test(unittest.TestCase):
                             {'null_comma': None}
                         }
 
+        self.unset_nml = {'unset_nml':
+                            {'x': None,
+                             'y': None
+                            }
+                         }
+
         self.types_nml = {'types_nml':
                             {'v_integer': 1,
                              'v_float': 1.0,
@@ -37,6 +44,7 @@ class Test(unittest.TestCase):
                              'v_null_interior': [1, 2, None, 4],
                              'v_null_end': [1, 2, 3, None],
                              'v_zero': [1, 0, 3],
+                             'v_stride': [1, None, 3, None, 5, None, 7],
                             }
                           }
 
@@ -55,6 +63,8 @@ class Test(unittest.TestCase):
 
         self.string_nml = {'string_nml':
                             {'str_basic': 'hello',
+                             'str_no_delim': 'hello',
+                             'str_no_delim_no_esc': "a''b",
                              'single_esc_delim': "a 'single' delimiter",
                              'double_esc_delim': 'a "double" delimiter',
                              'double_nested': "''x'' \"y\"",
@@ -128,7 +138,35 @@ class Test(unittest.TestCase):
                             {'v': 1.},
                           }
 
+        self.multiline_nml = {'multiline_nml':
+                                {'x': [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+                                       1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+                                       1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+                                       1, 1, 1, 1, 1, 1, 1, 1]
+                                }
+                             }
 
+        self.ext_token_nml = {'ext_token_nml':
+                                {'x': 1}
+                             }
+
+    # Support functions
+    def assert_file_equal(self, source_fname, target_fname):
+        with open(source_fname) as source:
+            with open(target_fname) as target:
+                source_str = source.read()
+                target_str = target.read()
+                self.assertEqual(source_str, target_str)
+
+    def assert_write(self, nml, target_fname):
+        tmp_fname = 'tmp.nml'
+        f90nml.write(nml, tmp_fname)
+        try:
+            self.assert_file_equal(tmp_fname, target_fname)
+        finally:
+            os.remove(tmp_fname)
+
+    # Tests
     def test_empty(self):
         test_nml = f90nml.read('empty.nml')
         self.assertEqual(self.empty_nml, test_nml)
@@ -139,6 +177,11 @@ class Test(unittest.TestCase):
         self.assertEqual(self.null_nml, test_nml)
         self.assert_write(test_nml, 'null_target.nml')
 
+    def test_unset(self):
+        test_nml = f90nml.read('unset.nml')
+        self.assertEqual(self.unset_nml, test_nml)
+        self.assert_write(test_nml, 'unset.nml')
+
     def test_types(self):
         test_nml = f90nml.read('types.nml')
         self.assertEqual(self.types_nml, test_nml)
@@ -189,18 +232,88 @@ class Test(unittest.TestCase):
         self.assertEqual(self.dollar_nml, test_nml)
         self.assert_write(test_nml, 'dollar_target.nml')
 
-    def assert_write(self, nml, target_fname):
+    def test_multiline(self):
+        test_nml = f90nml.read('multiline.nml')
+        self.assertEqual(self.multiline_nml, test_nml)
+        self.assert_write(test_nml, 'multiline.nml')
 
+    def test_ext_token(self):
+        test_nml = f90nml.read('ext_token.nml')
+        self.assertEqual(self.ext_token_nml, test_nml)
+
+    def test_write_existing_file(self):
         tmp_fname = 'tmp.nml'
-        f90nml.write(nml, tmp_fname)
+        open(tmp_fname, 'w').close()
+        test_nml = f90nml.read('empty.nml')
+        self.assertRaises(IOError, test_nml.write, tmp_fname)
+        os.remove(tmp_fname)
+
+    def test_pop_key(self):
+        test_nml = f90nml.read('empty.nml')
+        test_nml.pop('empty_nml')
+        self.assertEqual(test_nml, f90nml.namelist.NmlDict())
+
+    def test_patch(self):
+        patch_nml = f90nml.read('types_patch.nml')
+        f90nml.patch('types.nml', patch_nml, 'tmp.nml')
+        test_nml = f90nml.read('tmp.nml')
         try:
-            with open(tmp_fname) as tmp:
-                with open(target_fname) as target:
-                    tmp_str = tmp.read()
-                    target_str = target.read()
-                    self.assertEqual(tmp_str, target_str)
+            self.assertEqual(test_nml, patch_nml)
         finally:
-            os.remove(tmp_fname)
+            os.remove('tmp.nml')
+
+    def test_default_patch(self):
+        patch_nml = f90nml.read('types_patch.nml')
+        f90nml.patch('types.nml', patch_nml)
+        test_nml = f90nml.read('types.nml~')
+        try:
+            self.assertEqual(test_nml, patch_nml)
+        finally:
+            os.remove('types.nml~')
+
+    def test_no_selfpatch(self):
+        patch_nml = f90nml.read('types_patch.nml')
+        self.assertRaises(ValueError, f90nml.patch,
+                          'types.nml', patch_nml, 'types.nml')
+
+    def test_comment_patch(self):
+        nml = {'comment_nml': {'v_cmt_inline': 456}}
+        try:
+            f90nml.patch('comment.nml', nml, 'tmp.nml')
+            self.assert_file_equal('comment_patch.nml', 'tmp.nml')
+        finally:
+            os.remove('tmp.nml')
+
+    def test_index_syntax(self):
+        self.assertRaises(ValueError, f90nml.read, 'index_empty.nml')
+        self.assertRaises(ValueError, f90nml.read, 'index_bad.nml')
+        self.assertRaises(ValueError, f90nml.read, 'index_bad_start.nml')
+        self.assertRaises(ValueError, f90nml.read, 'index_empty_end.nml')
+        self.assertRaises(ValueError, f90nml.read, 'index_bad_end.nml')
+        self.assertRaises(ValueError, f90nml.read, 'index_empty_stride.nml')
+        self.assertRaises(ValueError, f90nml.read, 'index_bad_stride.nml')
+        self.assertRaises(ValueError, f90nml.read, 'index_zero_stride.nml')
+
+    def test_f90repr(self):
+        self.assertEqual(f90repr(1), '1')
+        self.assertEqual(f90repr(1.), '1.0')
+        self.assertEqual(f90repr(1+2j), '(1.0, 2.0)')
+        self.assertEqual(f90repr(True), '.true.')
+        self.assertEqual(f90repr(False), '.false.')
+        self.assertEqual(f90repr('abc'), "'abc'")
+
+        for ptype in ({}, [], set()):
+            self.assertRaises(ValueError, f90repr, ptype)
+
+    def test_pybool(self):
+        for fstr_true in ('true', 'ture', 't', '.t'):
+            self.assertEqual(pybool(fstr_true), True)
+
+        for fstr_false in ('false', 'flase', 'f', '.f'):
+            self.assertEqual(pybool(fstr_false), False)
+
+        for fstr in ('g', '.', 'xyz'):
+            self.assertRaises(ValueError, pybool, fstr)
 
 
 if __name__ == '__main__':
